@@ -48,6 +48,7 @@ static constexpr const char* DEFAULT_SYNTAX_C =
     "re2c:YYFILL:naked = 0;\n"
     "re2c:YYFN = [\"<undefined-function-name>;<undefined-function-type>\"];\n"
     "re2c:yyfn:sep = \";\";\n"
+    "re2c:yyfn:throw = \"\";\n"
     "re2c:yycond = \"yycond\";\n"
     "re2c:yyctable = \"yyctable\";\n"
     "re2c:yyaccept = \"yyaccept\";\n"
@@ -88,6 +89,7 @@ static constexpr const char* DEFAULT_SYNTAX_C =
     "re2c:debug-output = 0;\n"
     "re2c:computed-gotos = 0;\n"
     "re2c:computed-gotos:threshold = 9;\n"
+    "re2c:computed-gotos:relative = 0;\n"
     "re2c:nested-ifs = 0;\n"
     "re2c:case-insensitive = 0;\n"
     "re2c:case-inverted = 0;\n"
@@ -161,7 +163,7 @@ static constexpr const char* DEFAULT_SYNTAX_C =
     "code:const_global = topindent \"#define \" name \" \" init nl;\n"
     "\n"
     "code:array_local =\n"
-    "    topindent type \" \" name \"[\" size \"] = {\" nl indent\n"
+    "    topindent (.const ? \"static const \") type \" \" name \"[\" size \"] = {\" nl indent\n"
     "        [row{0:-2}: topindent [elem{0:-2}: elem \", \"] [elem{-1}: elem \",\"] nl]\n"
     "        [row{-1}:   topindent [elem{0:-2}: elem \", \"] [elem{-1}: elem]     nl]\n"
     "    dedent topindent \"};\" nl;\n"
@@ -182,8 +184,9 @@ static constexpr const char* DEFAULT_SYNTAX_C =
     "\n"
     "code:type_int = \"int\";\n"
     "code:type_uint = \"unsigned int\";\n"
-    "code:type_yybm = \"static const unsigned char\";\n"
-    "code:type_yytarget = \"static const void*\";\n"
+    "code:type_yybm = \"unsigned char\";\n"
+    "code:type_yytarget = (.cgoto.relative ? \"int\" : \"void*\");\n"
+    "code:type_yyctable = (.cgoto.relative ? \"int\" : \"void*\");\n"
     "\n"
     "code:cmp_eq = \"==\";\n"
     "code:cmp_ne = \"!=\";\n"
@@ -233,6 +236,16 @@ static constexpr const char* DEFAULT_SYNTAX_C =
     "\n"
     "code:goto = topindent \"goto \" label \";\" nl;\n"
     "\n"
+    "code:cgoto =\n"
+    "    topindent \"goto \" (.cgoto.relative\n"
+    "        ? \"*((char*)&&\" base \" + \" array \"[\" index \"])\"\n"
+    "        : \"*\" array \"[\" index \"]\"\n"
+    "    ) \";\" nl;\n"
+    "\n"
+    "code:cgoto_data = (.cgoto.relative\n"
+    "    ? \"(\" type \")((char*)&&\" label \" - (char*)&&\" base \")\"\n"
+    "    : \"&&\" label);\n"
+    "\n"
     "code:fndecl =\n"
     "    topindent type \" \" name \"(\"\n"
     "        [arg{0:-2}: argtype \" \" argname \", \"]\n"
@@ -441,4 +454,6 @@ static constexpr const char* DEFAULT_SYNTAX_C =
     "code:yybm_filter = yych \" & ~0xFF\";\n"
     "\n"
     "code:yybm_match = yybm \"[\" offset \"+\" yych \"] & \" mask;\n"
+    "\n"
+    "code:yytarget_filter = yych \" & ~0xFF\";\n"
     ;
