@@ -48,6 +48,7 @@ static constexpr const char* DEFAULT_SYNTAX_OCAML =
     "re2c:YYFILL:naked = 0;\n"
     "re2c:YYFN = [\"<undefined-function-name>;<undefined-function-type>\"];\n"
     "re2c:yyfn:sep = \";\";\n"
+    "re2c:yyfn:throw = \"\";\n"
     "re2c:yycond = \"yycond\";\n"
     "re2c:yyctable = \"yyctable\";\n"
     "re2c:yyaccept = \"yyaccept\";\n"
@@ -88,6 +89,7 @@ static constexpr const char* DEFAULT_SYNTAX_OCAML =
     "re2c:debug-output = 0;\n"
     "re2c:computed-gotos = 0;\n"
     "re2c:computed-gotos:threshold = 9;\n"
+    "re2c:computed-gotos:relative = 0;\n"
     "re2c:nested-ifs = 0;\n"
     "re2c:case-insensitive = 0;\n"
     "re2c:case-inverted = 0;\n"
@@ -183,6 +185,7 @@ static constexpr const char* DEFAULT_SYNTAX_OCAML =
     "code:type_uint = \"uint\";\n"
     "code:type_yybm = \"int\";\n"
     "code:type_yytarget = <undefined>;\n"
+    "code:type_yyctable = <undefined>;\n"
     "\n"
     "code:cmp_eq = \"==\"; // physical equality, as this is used to compare primitive values\n"
     "code:cmp_ne = \"!=\"; // physical inequality, as this is used to compare primitive values\n"
@@ -228,10 +231,10 @@ static constexpr const char* DEFAULT_SYNTAX_OCAML =
     "code:switch_case_default = \"_\";\n"
     "\n"
     "code:loop = <undefined>;\n"
-    "\n"
     "code:continue = <undefined>;\n"
-    "\n"
     "code:goto = <undefined>;\n"
+    "code:cgoto = <undefined>;\n"
+    "code:cgoto_data = <undefined>;\n"
     "\n"
     "code:fndecl = <undefined>;\n"
     "\n"
@@ -394,4 +397,6 @@ static constexpr const char* DEFAULT_SYNTAX_OCAML =
     "code:yybm_filter = yych \" land ~0xFF\";\n"
     "\n"
     "code:yybm_match = \"(\" yybm \".(\" offset \" + Char.code \" yych \") land \" mask \") != 0\";\n"
+    "\n"
+    "code:yytarget_filter = <undefined>;\n"
     ;
