@@ -48,6 +48,7 @@ static constexpr const char* DEFAULT_SYNTAX_PYTHON =
     "re2c:YYFILL:naked = 0;\n"
     "re2c:YYFN = [\"<undefined-function-name>;<undefined-function-type>\"];\n"
     "re2c:yyfn:sep = \";\";\n"
+    "re2c:yyfn:throw = \"\";\n"
     "re2c:yycond = \"yycond\";\n"
     "re2c:yyctable = \"yyctable\";\n"
     "re2c:yyaccept = \"yyaccept\";\n"
@@ -88,6 +89,7 @@ static constexpr const char* DEFAULT_SYNTAX_PYTHON =
     "re2c:debug-output = 0;\n"
     "re2c:computed-gotos = 0;\n"
     "re2c:computed-gotos:threshold = 9;\n"
+    "re2c:computed-gotos:relative = 0;\n"
     "re2c:nested-ifs = 1;\n"
     "re2c:case-insensitive = 0;\n"
     "re2c:case-inverted = 0;\n"
@@ -179,6 +181,7 @@ static constexpr const char* DEFAULT_SYNTAX_PYTHON =
     "code:type_uint = <undefined>;\n"
     "code:type_yybm = <undefined>;\n"
     "code:type_yytarget = <undefined>;\n"
+    "code:type_yyctable = <undefined>;\n"
     "\n"
     "code:cmp_eq = \"==\";\n"
     "code:cmp_ne = \"!=\";\n"
@@ -219,6 +222,8 @@ static constexpr const char* DEFAULT_SYNTAX_PYTHON =
     "code:continue = topindent \"continue\" nl;\n"
     "\n"
     "code:goto = <undefined>;\n"
+    "code:cgoto = <undefined>;\n"
+    "code:cgoto_data = <undefined>;\n"
     "\n"
     "code:fndecl = <undefined>;\n"
     "\n"
@@ -387,4 +392,6 @@ static constexpr const char* DEFAULT_SYNTAX_PYTHON =
     "code:yybm_filter = yych \" & ~0xFF\";\n"
     "\n"
     "code:yybm_match = \"(\" yybm \"[\" offset \"+\" yych \"] & \" mask \") != 0\";\n"
+    "\n"
+    "code:yytarget_filter = <undefined>;\n"
     ;
