@@ -48,6 +48,7 @@ static constexpr const char* DEFAULT_SYNTAX_V =
     "re2c:YYFILL:naked = 0;\n"
     "re2c:YYFN = [\"<undefined-function-name>;<undefined-function-type>\"];\n"
     "re2c:yyfn:sep = \";\";\n"
+    "re2c:yyfn:throw = \"\";\n"
     "re2c:yycond = \"yycond\";\n"
     "re2c:yyctable = \"yyctable\";\n"
     "re2c:yyaccept = \"yyaccept\";\n"
@@ -88,6 +89,7 @@ static constexpr const char* DEFAULT_SYNTAX_V =
     "re2c:debug-output = 0;\n"
     "re2c:computed-gotos = 0;\n"
     "re2c:computed-gotos:threshold = 9;\n"
+    "re2c:computed-gotos:relative = 0;\n"
     "re2c:nested-ifs = 0;\n"
     "re2c:case-insensitive = 0;\n"
     "re2c:case-inverted = 0;\n"
@@ -160,6 +162,7 @@ static constexpr const char* DEFAULT_SYNTAX_V =
     "code:const_local = topindent \"const \" name \" = \" init nl;\n"
     "code:const_global = code:const_local;\n"
     "\n"
+    "// V has only top-level constants, so all local arrays are mutable.\n"
     "code:array_local =\n"
     "    topindent name \" := [\" nl indent\n"
     "        [row: topindent [elem{0:-2}: elem \", \"] [elem{-1}: elem \",\"] nl]\n"
@@ -184,6 +187,7 @@ static constexpr const char* DEFAULT_SYNTAX_V =
     "code:type_uint = \"u32\";\n"
     "code:type_yybm = \"u8\";\n"
     "code:type_yytarget = <undefined>;\n"
+    "code:type_yyctable = <undefined>;\n"
     "\n"
     "code:cmp_eq = \"==\";\n"
     "code:cmp_ne = \"!=\";\n"
@@ -230,6 +234,9 @@ static constexpr const char* DEFAULT_SYNTAX_V =
     "\n"
     "code:goto = topindent \"unsafe { goto \" label \" }\" nl;\n"
     "\n"
+    "code:cgoto = <undefined>;\n"
+    "code:cgoto_data = <undefined>;\n"
+    "\n"
     "code:fndecl = <undefined>;\n"
     "\n"
     "code:fndef =\n"
@@ -404,4 +411,6 @@ static constexpr const char* DEFAULT_SYNTAX_V =
     "code:yybm_filter = yych \" & ~0xFF\";\n"
     "\n"
     "code:yybm_match = \"(\" yybm \"[\" offset \"+\" yych \"] & \" mask \") != 0\";\n"
+    "\n"
+    "code:yytarget_filter = <undefined>;\n"
     ;
