@@ -48,6 +48,7 @@ static constexpr const char* DEFAULT_SYNTAX_ZIG =
     "re2c:YYFILL:naked = 0;\n"
     "re2c:YYFN = [\"<undefined-function-name>;<undefined-function-type>\"];\n"
     "re2c:yyfn:sep = \";\";\n"
+    "re2c:yyfn:throw = \"\";\n"
     "re2c:yycond = \"yycond\";\n"
     "re2c:yyctable = \"yyctable\";\n"
     "re2c:yyaccept = \"yyaccept\";\n"
@@ -88,6 +89,7 @@ static constexpr const char* DEFAULT_SYNTAX_ZIG =
     "re2c:debug-output = 0;\n"
     "re2c:computed-gotos = 0;\n"
     "re2c:computed-gotos:threshold = 9;\n"
+    "re2c:computed-gotos:relative = 0;\n"
     "re2c:nested-ifs = 0;\n"
     "re2c:case-insensitive = 0;\n"
     "re2c:case-inverted = 0;\n"
@@ -161,7 +163,7 @@ static constexpr const char* DEFAULT_SYNTAX_ZIG =
     "code:const_global = code:const_local;\n"
     "\n"
     "code:array_local =\n"
-    "    topindent \"const \" name \" = [\" size \"]\" type \"{\" nl indent\n"
+    "    topindent (.const ? \"const \") name \" = [\" size \"]\" type \"{\" nl indent\n"
     "        [row: topindent [elem{0:-2}: elem \", \"] [elem{-1}: elem \",\"] nl]\n"
     "    dedent topindent \"};\" nl;\n"
     "\n"
@@ -179,6 +181,7 @@ static constexpr const char* DEFAULT_SYNTAX_ZIG =
     "code:type_uint = \"u32\";\n"
     "code:type_yybm = \"u8\";\n"
     "code:type_yytarget = <undefined>;\n"
+    "code:type_yyctable = <undefined>;\n"
     "\n"
     "code:cmp_eq = \"==\";\n"
     "code:cmp_ne = \"!=\";\n"
@@ -226,21 +229,26 @@ static constexpr const char* DEFAULT_SYNTAX_ZIG =
     "\n"
     "code:goto = topindent \"goto \" label \";\" nl;\n"
     "\n"
+    "code:cgoto = <undefined>;\n"
+    "code:cgoto_data = <undefined>;\n"
+    "\n"
     "code:fndecl = <undefined>;\n"
     "\n"
     "code:fndef =\n"
     "    topindent \"fn \" name \"(\"\n"
     "        [arg{0:-2}: argname \": \" argtype \", \"]\n"
     "        [arg{-1}:   argname \": \" argtype]\n"
-    "    \") \" type \" {\" nl\n"
+    "    \") \" (.yyfn.throw ? throw \"!\") type \" {\" nl\n"
     "        indent [stmt: stmt] dedent\n"
     "    \"}\" nl;\n"
     "\n"
     "code:fncall =\n"
-    "    topindent (.retval ? retval \" = \" name) \"(\" [arg{0:-2}: arg \", \"] [arg{-1}: arg] \");\" nl;\n"
+    "    topindent (.retval ? retval \" = \") (.yyfn.throw ? \"try \")\n"
+    "        name \"(\" [arg{0:-2}: arg \", \"] [arg{-1}: arg] \");\" nl;\n"
     "\n"
     "code:tailcall =\n"
-    "    topindent (.retval ? \"return \" name) \"(\" [arg{0:-2}: arg \", \"] [arg{-1}: arg] \");\" nl;\n"
+    "    topindent (.retval ? \"return \") (.yyfn.throw ? \"try \")\n"
+    "        name \"(\" [arg{0:-2}: arg \", \"] [arg{-1}: arg] \");\" nl;\n"
     "\n"
     "code:recursive_functions = [fn: fndef nl];\n"
     "\n"
@@ -417,4 +425,6 @@ static constexpr const char* DEFAULT_SYNTAX_ZIG =
     "code:yybm_filter = yych \" & ~0xFF\";\n"
     "\n"
     "code:yybm_match = \"(\" yybm \"[\" offset \"+\" yych \"] & \" mask \") != 0\";\n"
+    "\n"
+    "code:yytarget_filter = <undefined>;\n"
     ;
