@@ -489,6 +489,14 @@ const char* help =
 "\n"
 "        Warn about rules that are shadowed by other rules and will never match.\n"
 "\n"
+"    -Wdeprecated-eof_rule\n"
+"\n"
+"        Warn about standalone end of input rules $ that will be broken by the\n"
+"        future changes and require fixing. At the moment these rules take\n"
+"        precedence when conflicting with other rules, but after the\n"
+"        introduction of generalized end of input symbol $ precedence order will\n"
+"        change and these rules will become shadowed by other rules.\n"
+"\n"
 "    -Wuseless-escape\n"
 "\n"
 "        Warn if a symbol is escaped when it shouldn't be. By default, re2py\n"
