@@ -55,7 +55,7 @@ const char* help =
 "        supported by compilers like GCC, Clang and Tcc. The main advantage over\n"
 "        using single cases is smaller generated code and faster generation\n"
 "        time, although for some compilers like Tcc it also results in smaller\n"
-"        binary size. This option is supported only for C.\n"
+"        binary size.\n"
 "\n"
 "    --computed-gotos -g\n"
 "\n"
@@ -63,8 +63,14 @@ const char* help =
 "        (which must be supported by the compiler). re2rust generates jump\n"
 "        tables only in complex cases with a lot of conditional branches.\n"
 "        Complexity threshold can be configured with cgoto:threshold\n"
-"        configuration. This option implies --bit-vectors. It is supported only\n"
-"        for C.\n"
+"        configuration. Relative offsets can be enabled with cgoto:relative\n"
+"        configuration. This option implies --bit-vectors.\n"
+"\n"
+"    --computed-gotos-relative\n"
+"\n"
+"        Similar to --computed-gotos but generate relative offsets for jump\n"
+"        tables instead (which must be supported by the compiler). This option\n"
+"        implies --computed-gotos.\n"
 "\n"
 "    --conditions --start-conditions -c\n"
 "\n"
@@ -148,13 +154,13 @@ const char* help =
 "        (...) is capturing and (! ...) is non-capturing. With this option (!\n"
 "        ...) is capturing and (...) is non-capturing.\n"
 "\n"
-"    --lang <none | c | d | go | haskell | java | js | ocaml | python | rust | v\n"
-"    | zig>\n"
+"    --lang <none | c | d | go | haskell | java | js | ocaml | python | rust |\n"
+"    swift | v | zig>\n"
 "\n"
 "        Specify the target language. Supported languages are C, D, Go, Haskell,\n"
-"        Java, JS, OCaml, Python, Rust, V, Zig (more languages can be added via\n"
-"        user-defined syntax files, see the --syntax option). Option none\n"
-"        disables default suntax configs, so that the target language is\n"
+"        Java, JS, OCaml, Python, Rust, Swift, V, Zig (more languages can be\n"
+"        added via user-defined syntax files, see the --syntax option). Option\n"
+"        none disables default suntax configs, so that the target language is\n"
 "        undefined.\n"
 "\n"
 "    --location-format <gnu | msvc>\n"
@@ -484,6 +490,14 @@ const char* help =
 "\n"
 "        Warn about rules that are shadowed by other rules and will never match.\n"
 "\n"
+"    -Wdeprecated-eof_rule\n"
+"\n"
+"        Warn about standalone end of input rules $ that will be broken by the\n"
+"        future changes and require fixing. At the moment these rules take\n"
+"        precedence when conflicting with other rules, but after the\n"
+"        introduction of generalized end of input symbol $ precedence order will\n"
+"        change and these rules will become shadowed by other rules.\n"
+"\n"
 "    -Wuseless-escape\n"
 "\n"
 "        Warn if a symbol is escaped when it shouldn't be. By default, re2rust\n"
