@@ -133,15 +133,15 @@ struct CodeGoIf {
     } kind;
 };
 
-struct CodeGoCpTable {
+struct CodeGoCgotoTable {
     static constexpr uint32_t TABLE_SIZE = 0x100;
 
     State** table;
 };
 
-struct CodeGoCp {
+struct CodeGoCgoto {
     CodeGoSwIf* hgo;
-    CodeGoCpTable* table;
+    CodeGoCgotoTable* table;
 };
 
 struct CodeGo {
@@ -149,7 +149,7 @@ struct CodeGo {
         EMPTY,
         SWITCH_IF,
         LINEAR_IF,
-        CPGOTO,
+        CGOTO,
         DOT
     };
 
@@ -160,7 +160,7 @@ struct CodeGo {
     bool skip;
     union {
         CodeGoSwIf* goswif;
-        CodeGoCp* gocp;
+        CodeGoCgoto* cgoto;
         CodeGoSw* godot;
         CodeGoIfL* goifl;
     };
@@ -238,6 +238,12 @@ struct CodeRaw {
     size_t size;
 };
 
+struct CodeCgoto {
+    const char* array;
+    const char* base;
+    const char* index;
+};
+
 struct CodeParam {
     const char* name;
     const char* type;
@@ -285,6 +291,7 @@ struct CodeArray {
     const char** elems;
     size_t size;
     bool local;
+    bool constant;
     bool tabulate;
 };
 
@@ -315,6 +322,7 @@ struct Code {
         CodeBranches* ifte;
         CodeSwitch swch;
         CodeBlock block;
+        CodeCgoto cgoto;
         CodeFnDef fndef;
         CodeFnCall fncall;
         CodeRaw raw;
@@ -366,6 +374,14 @@ inline Code* code_goto(OutAllocator& alc, const char* target) {
     return x;
 }
 
+inline Code* code_cgoto(OutAllocator& alc, const char* array, const char* base, const char* index) {
+    Code* x = new_code(alc, CodeKind::CGOTO);
+    x->cgoto.array = array;
+    x->cgoto.base = base;
+    x->cgoto.index = index;
+    return x;
+}
+
 inline Code* code_continue(OutAllocator& alc, const char* target) {
     Code* x = new_code(alc, CodeKind::CONTINUE);
     x->target = target;
@@ -762,6 +778,7 @@ inline Code* code_array(
         const char** elems,
         size_t size,
         bool local,
+        bool constant,
         bool tabulate = false) {
     Code* x = new_code(alc, CodeKind::ARRAY);
     x->array.name = name;
@@ -769,6 +786,7 @@ inline Code* code_array(
     x->array.elems = elems;
     x->array.size = size;
     x->array.local = local;
+    x->array.constant = constant;
     x->array.tabulate = tabulate;
     return x;
 }
