@@ -177,6 +177,52 @@ class GenEnumElem : public RenderCallback {
     FORBID_COPY(GenEnumElem);
 };
 
+template <typename base_t, typename label_t>
+class GenCgotoData : public RenderCallback {
+    std::ostream& os;
+    const char* type;
+    const std::string& prefix;
+    base_t base;
+    label_t label;
+
+  public:
+    GenCgotoData(std::ostream& os,
+            const char* type, const std::string& prefix, base_t base, label_t label)
+        : os(os), type(type), prefix(prefix), base(base), label(label) {}
+
+    void render_var(StxVarId var) override {
+        switch (var) {
+            case StxVarId::TYPE: os << type; break;
+            case StxVarId::BASE: os << prefix << base; break;
+            case StxVarId::LABEL: os << prefix << label; break;
+            default: UNREACHABLE(); break;
+        }
+    }
+
+    FORBID_COPY(GenCgotoData);
+};
+
+class GenTargetFilter : public RenderCallback {
+    std::ostream& os;
+    const opt_t* opts;
+
+  public:
+    GenTargetFilter(std::ostream& os, const opt_t* opts): os(os), opts(opts) {}
+
+    void render_var(StxVarId var) override {
+        switch (var) {
+        case StxVarId::CHAR:
+            os << opts->var_char;
+            break;
+        default:
+            UNREACHABLE();
+            break;
+        }
+    }
+
+    FORBID_COPY(GenTargetFilter);
+};
+
 } // anonymous namespace
 
 bool endstate(const State* s) {
@@ -754,45 +800,49 @@ static CodeList* gen_goswif(
            : gen_goif(output, dfa, go->goif, from);
 }
 
-static CodeList* gen_gocp_table(Output& output, const CodeGoCpTable* go) {
+static CodeList* gen_cgoto(
+        Output& output, const Adfa& dfa, const CodeGoCgoto* go, const State* from) {
     const opt_t* opts = output.block().opts;
     OutAllocator& alc = output.allocator;
     Scratchbuf& buf = output.scratchbuf;
+    State** targets = go->table->table;
 
-    const char** elems = alc.alloct<const char*>(CodeGoCpTable::TABLE_SIZE);
-    for (uint32_t i = 0; i < CodeGoCpTable::TABLE_SIZE; ++i) {
-        elems[i] = buf.cstr("&&").str(opts->label_prefix).u32(go->table[i]->label->index).flush();
+    CodeList* cgoto = code_list(alc);
+
+    uint32_t min_label = ~0u;
+    if (opts->computed_gotos_relative) {
+        for (uint32_t i = 0; i < CodeGoCgotoTable::TABLE_SIZE; ++i) {
+            min_label = std::min(targets[i]->label->index, min_label);
+        }
     }
 
-    const char* name = opts->var_computed_gotos_table.c_str();
+    const char* table = opts->var_cgoto_table.c_str();
     const char* type = opts->gen_code_type_yytarget(buf);
-    // In rec/func mode tables are reused across different fnctions, so they must be global.
+
+    const char** elems = alc.alloct<const char*>(CodeGoCgotoTable::TABLE_SIZE);
+    for (uint32_t i = 0; i < CodeGoCgotoTable::TABLE_SIZE; ++i) {
+        GenCgotoData<uint32_t, uint32_t> callback(
+            buf.stream(), type, opts->label_prefix, min_label, targets[i]->label->index);
+        elems[i] = opts->gen_code_cgoto_data(buf, callback);
+    }
+
+    // In rec/func mode tables are reused across different functions, so they must be global.
     bool local = opts->code_model != CodeModel::REC_FUNC;
 
-    CodeList* stmts = code_list(alc);
-    append(stmts, code_array(
-            alc, name, type, elems, CodeGoCpTable::TABLE_SIZE, local, /*tabulate*/ true));
-    return stmts;
-}
+    append(cgoto, code_array(alc, table, type, elems, CodeGoCgotoTable::TABLE_SIZE,
+        local, /*constant*/ true, /*tabulate*/ true));
 
-static CodeList* gen_gocp(Output& output, const Adfa& dfa, const CodeGoCp* go, const State* from) {
-    const opt_t* opts = output.block().opts;
-    OutAllocator& alc = output.allocator;
-    Scratchbuf& buf = output.scratchbuf;
+    const char* base = buf.str(opts->label_prefix).u32(min_label).flush();
+    append(cgoto, code_cgoto(alc, opts->var_cgoto_table.c_str(), base, opts->var_char.c_str()));
 
     CodeList* stmts = code_list(alc);
-
-    CodeList* if_else = gen_gocp_table(output, go->table);
-    buf.cstr("*").str(opts->var_computed_gotos_table).cstr("[").str(opts->var_char).cstr("]");
-    append(if_else, code_goto(alc, buf.flush()));
-
     if (go->hgo != nullptr) {
-        const char* cond =
-                buf.str(opts->var_char).cstr(" & ~0xFF").flush();
-        CodeList* if_then = gen_goswif(output, dfa, go->hgo, from);
-        append(stmts, code_if_then_else(alc, cond, if_then, if_else));
+        GenTargetFilter callback(buf.stream(), opts);
+        const char* cond = opts->gen_code_yytarget_filter(buf, callback);
+        CodeList* other = gen_goswif(output, dfa, go->hgo, from);
+        append(stmts, code_if_then_else(alc, cond, other, cgoto));
     } else {
-        append(stmts, code_block(alc, if_else, CodeBlock::Kind::WRAPPED));
+        append(stmts, code_block(alc, cgoto, CodeBlock::Kind::WRAPPED));
     }
 
     return stmts;
@@ -863,8 +913,8 @@ static void gen_go(
         append(stmts, gen_goswif(output, dfa, go->goswif, from));
     } else if (go->kind == CodeGo::Kind::LINEAR_IF) {
         append(stmts, gen_goifl(output, dfa, go->goifl, from));
-    } else if (go->kind == CodeGo::Kind::CPGOTO) {
-        append(stmts, gen_gocp(output, dfa, go->gocp, from));
+    } else if (go->kind == CodeGo::Kind::CGOTO) {
+        append(stmts, gen_cgoto(output, dfa, go->cgoto, from));
     }
 }
 
@@ -920,19 +970,28 @@ static void emit_accept(
     if (opts->computed_gotos && nacc >= opts->computed_gotos_threshold && !have_tags) {
         CodeList* block = code_list(alc);
 
-        const char** elems = alc.alloct<const char*>(nacc);
-        for (uint32_t i = 0; i < nacc; ++i) {
-            buf.cstr("&&").str(opts->label_prefix).u32(acc[i].state->label->index);
-            elems[i] = buf.flush();
+        uint32_t min_label = ~0u;
+        if (opts->computed_gotos_relative) {
+            for (uint32_t i = 0; i < nacc; ++i) {
+                min_label = std::min(acc[i].state->label->index, min_label);
+            }
         }
-        const char* name = opts->var_computed_gotos_table.c_str();
+
+        const char* name = opts->var_cgoto_table.c_str();
         const char* type = opts->gen_code_type_yytarget(buf);
 
-        // In rec/func mode the table can be local, as it's used in the same function.
-        append(block, code_array(alc, name, type, elems, nacc, /*local*/ true));
+        const char** elems = alc.alloct<const char*>(nacc);
+        for (uint32_t i = 0; i < nacc; ++i) {
+            GenCgotoData<uint32_t, uint32_t> callback(
+                buf.stream(), type, opts->label_prefix, min_label, acc[i].state->label->index);
+            elems[i] = opts->gen_code_cgoto_data(buf, callback);
+        }
 
-        buf.cstr("*").str(opts->var_computed_gotos_table).cstr("[").cstr(var).cstr("]");
-        append(block, code_goto(alc, buf.flush()));
+        // In rec/func mode the table can be local, as it's used in the same function.
+        append(block, code_array(alc, name, type, elems, nacc, /*local*/ true, /*constant*/ true));
+
+        const char* base = buf.str(opts->label_prefix).u32(min_label).flush();
+        append(block, code_cgoto(alc, opts->var_cgoto_table.c_str(), base, var));
 
         append(stmts, code_block(alc, block, CodeBlock::Kind::WRAPPED));
         return;
@@ -1571,8 +1630,8 @@ static CodeList* gen_cond_goto(Output& output) {
     const char* getcond = opts->gen_code_yygetcond(buf, callback);
 
     if (opts->computed_gotos) {
-        buf.cstr("*").str(opts->var_cond_table).cstr("[").cstr(getcond).cstr("]");
-        append(stmts, code_goto(alc, buf.flush()));
+        const char* base = buf.str(opts->cond_label_prefix).str(conds[0].name).flush();
+        append(stmts, code_cgoto(alc, opts->var_cond_table.c_str(), base, getcond));
     } else if (opts->nested_ifs) {
         warn_cond_ord &= ncond > 1;
         append(stmts, gen_cond_goto_binary(output, getcond, 0, ncond - 1));
@@ -1585,8 +1644,7 @@ static CodeList* gen_cond_goto(Output& output) {
             buf.str(opts->cond_label_prefix).str(cond.name);
             append(body, code_goto(alc, buf.flush()));
 
-            append(ccases, code_case_string(alc, body,
-                    gen_cond_enum_elem(buf, opts, cond.name)));
+            append(ccases, code_case_string(alc, body, gen_cond_enum_elem(buf, opts, cond.name)));
         }
         if (opts->cond_abort) {
             append(ccases, code_case_default(alc, gen_abort(alc)));
@@ -1602,6 +1660,7 @@ static CodeList* gen_cond_goto(Output& output) {
 
     return stmts;
 }
+
 static CodeList* gen_cond_table(Output& output) {
     const opt_t* opts = output.block().opts;
     OutAllocator& alc = output.allocator;
@@ -1610,15 +1669,19 @@ static CodeList* gen_cond_table(Output& output) {
 
     CodeList* code = code_list(alc);
 
-    const char** elems = alc.alloct<const char*>(conds.size());
-    for (size_t i = 0; i < conds.size(); ++i) {
-        elems[i] = buf.cstr("&&").str(opts->cond_label_prefix).str(conds[i].name).flush();
-    }
     const char* name = opts->var_cond_table.c_str();
-    const char* type = opts->gen_code_type_yytarget(buf);
+    const char* type = opts->gen_code_type_yyctable(buf);
+
+    const char** elems = alc.alloct<const char*>(conds.size());
+    for (uint32_t i = 0; i < conds.size(); ++i) {
+        GenCgotoData<const std::string&, const std::string&> callback(
+                buf.stream(), type, opts->cond_label_prefix, conds[0].name, conds[i].name);
+        elems[i] = opts->gen_code_cgoto_data(buf, callback);
+    }
 
     // In rec/func mode the table can be local, as it's used in the same function.
-    append(code, code_array(alc, name, type, elems, conds.size(), /*local*/ true));
+    append(code, code_array(
+        alc, name, type, elems, conds.size(), /*local*/ true, /*constant*/ true));
     return code;
 }
 
@@ -1713,8 +1776,8 @@ CodeList* gen_bitmap(Output& output, const CodeBitmap* bitmap, const std::string
     bool local = opts->code_model != CodeModel::REC_FUNC;
 
     CodeList* stmts = code_list(alc);
-    append(stmts, code_array(
-            alc, name, type, bitmap->elems, bitmap->nelems, local, /*tabulate*/ true));
+    append(stmts, code_array(alc, name, type, bitmap->elems, bitmap->nelems,
+        local, /*constant*/ true, /*tabulate*/ true));
     return stmts;
 }
 
