@@ -22,6 +22,7 @@ enum class Lang: uint32_t {
     OCAML,
     PYTHON,
     RUST,
+    SWIFT,
     V,
     ZIG,
     NONE // no language (no default syntax config will be loaded)
@@ -136,6 +137,7 @@ enum class CodeKind: uint32_t {
     ENUM,
     STMT,
     GOTO,
+    CGOTO,
     CONTINUE,
     LOOP,
     TEXT,
@@ -201,12 +203,15 @@ enum class StxGOpt {
     CASE_RANGES,
     UNSAFE,
     MONADIC,
-    LOOP_LABEL
+    LOOP_LABEL,
+    CGOTO_RELATIVE,
+    FN_THROW
 };
 
 enum class StxLOpt {
     CAST,
     CHAR_LITERALS,
+    CONST,
     ARGS,
     COND,
     INIT,
