@@ -73,6 +73,18 @@ Input::~Input() {
     }
 }
 
+void Input::reset_lexer() {
+    reset_ptrs();
+
+    for (InputFile* in: files) {
+        delete in;
+    }
+    files.clear();
+    msg.filenames.clear();
+
+    location = ATSTART;
+}
+
 size_t Input::get_input_index() const {
     // Find index of the current input file: the one corresponding to the buffer fragment that
     // contains the cursor.
@@ -100,14 +112,14 @@ Ret Input::include(const std::string& filename, uint8_t* at) {
     // second time when it has been fully read. The second time is needed to generate a line
     // directive marking the end of the include file and the continuation of the parent file. In
     // order to make the lexer stop at the boundary between files and do the second call, we
-    // "unread" the directive on the first call (push it back to the parent file before filling the
-    // buffer with the contents of include file), and let the lexer scan the directive twice.
+    // "unread" the `include` block on the first call (push it back to the parent file before
+    // filling the buffer with the contents of include file), and let the lexer scan it twice.
     //
     // To avoid infinite recursion we need to differentiate the two calls. We do this by inspecting
     // the files on stack. First, we pop all finished files: there may be sibling includes before
     // this one in the parent file. They must be finished by now, meaning that the token pointer has
-    // moved past their end pointer (even if another directive immediately precedes this one,
-    // "unreading" the directive has the effect of inserting an artificial boundary between include
+    // moved past their end pointer (even if another `include` block immediately precedes this one,
+    // "unreading" the block has the effect of inserting an artificial boundary between include
     // files). However if the current include file is already on stack, then `at` must point exactly
     // at the end of it (not past the end), so we will not pop it.
     //
@@ -119,7 +131,7 @@ Ret Input::include(const std::string& filename, uint8_t* at) {
     CHECK(tok <= at); // ensure that we won't pop the include file itself
     pop_finished_files();
 
-    const size_t fidx = get_input_index(); // index of the parent file with the directive
+    const size_t fidx = get_input_index(); // index of the parent file with the `include` block
     const size_t last = files.size() - 1; // index of the topmost file
     if (fidx == last) {
         // This is the first call, go on.
@@ -138,7 +150,7 @@ Ret Input::include(const std::string& filename, uint8_t* at) {
     // files. In the file stack, however, outer files go before nested files (nested are at the
     // top). We want to break from the unreading cycle early, therefore we go in reverse order of
     // file offsets in buffer and break as soon as the end offset is less than cursor (current
-    // position). `at` points at the start of include directive.
+    // position). `at` points at the start of `include` block.
     for (InputFile* in : files) {
         if (in->so >= at) {
             // unread whole fragment
@@ -168,10 +180,22 @@ bool Input::read(size_t want) {
     for (size_t i = files.size(); i --> 0; ) {
         InputFile* in = files[i];
         const size_t have = fread(lim, 1, want, in->file);
-        in->so = lim;
-        lim += have;
-        in->eo = lim;
-        want -= have;
+        if (have > 0) {
+            in->so = lim;
+            lim += have;
+            in->eo = lim;
+            want -= have;
+        } else if (in->so == ENDPOS) {
+            // Empty file: first time we read from it and have nothing.
+            DCHECK(in->eo == ENDPOS);
+            in->so = in->eo = lim;
+        } else {
+            // We have reached the end of file. Don't adjust start and end pointers, as
+            // YYLIMIT may point outside of this file. This may happen if the previous `read()`
+            // invocation drained this file, then read some data from the parent file, and
+            // then YYFILL got triggered while YYCURSOR was still in this file, so this file
+            // could not be popped off the stack yet.
+        }
 
         // buffer filled
         if (want == 0) return true;
@@ -223,11 +247,7 @@ bool Input::fill(size_t need) {
         shift_ptrs_and_fpos(-static_cast<ptrdiff_t>(free));
     } else {
         BSIZE += std::max(BSIZE, need);
-        uint8_t* buf = new uint8_t[BSIZE + YYMAXFILL];
-        if (buf == nullptr) {
-            error("out of memory");
-            return false;
-        }
+        uint8_t* buf = new uint8_t[BSIZE + maxfill()];
 
         memmove(buf, tok, copy);
         shift_ptrs_and_fpos(buf - tok);
@@ -240,8 +260,8 @@ bool Input::fill(size_t need) {
     CHECK(lim + free <= bot + BSIZE);
     if (!read(free)) {
         eof = lim;
-        memset(lim, 0, YYMAXFILL);
-        lim += YYMAXFILL;
+        memset(lim, 0, maxfill());
+        lim += maxfill();
     }
 
     return true;
