@@ -1,13 +1,14 @@
 #ifndef _RE2C_UTIL_FILE_UTILS_
 #define _RE2C_UTIL_FILE_UTILS_
 
+#include <stdio.h>
 #include <string>
 
-
 namespace re2c {
 
-void get_dir(std::string &path);
-std::string escape_backslashes(const std::string &str);
+FILE* temp_file(std::string& fname);
+bool overwrite_file(const char* srcname, const char* dstname);
+void get_dir(std::string& path);
 
 } // namespace re2c
 
